"""Console script for p2_cli."""
import os
import re
import sys
import click
import pprint
import time
import json
from tabulate import tabulate
from datetime import date, datetime

from p2.store.client import Client

p2_store_endpoint = os.environ.get('P2_STORE_URL')

if p2_store_endpoint is None:
    click.echo("P2_STORE_URL not set")
    exit()

pp = pprint.PrettyPrinter(indent=2, width=140)
pp_col = pprint.PrettyPrinter(indent=2, width=40)
client = Client( p2_store_endpoint )

@click.group()
def cli():
    pass

@cli.command(help="Get component configuration")
def list_config(args=None):
    data = client.select(
        table='config',
        fetch=['component', 'jsonb_pretty(config) as config']
    )
    click.echo(tabulate(data, headers='keys'))

@cli.command(help="Lists all experiments ")
def list_experiments(args=None ):
    data = client.get_experiments()
    if data is None:
        click.echo("Error getting experiments")
    else:
        click.echo("Experiments")
        click.echo('-'*80)
        for experiment in data:
            click.echo(experiment['filename'])
    return 0

@cli.command(help="Returns the selected experiment")
@click.option('--filename', required=True, help='the experiment filename')
def get_experiment(filename):
    click.echo("Filename: {}".format(filename))
    click.echo('-'*80)
    data   = client.get_experiment(filename)
    if data is None:
        click.echo("Error retrieving data for experiment {}".format(filename))
    else:
        pp.pprint(data)

@cli.command(help="Loads an experiment")
@click.option('--filename', required=True, help='the experiment filename to load')
def add_experiment(filename):
    with open(filename) as f:
        data = json.load(f)

        ret = client.add_experiment('-'.join(filename.split('.')[0:-1]), data)
        if ret is not None:
            click.echo("Experiment loaded")
        else:
            click.echo("Error loading experiment")

@cli.command(help="Loads a template")
@click.option('--filename', required=True, help='the template filename to load')
@click.option('--params', required=True, help='the template params (JSON) filename to load')
@click.option('--tag', help='A tag to associate with the template')
def add_template(filename, params, tag):
    if tag:
        click.echo(f"Adding template {filename} with tag {tag}...")
    else:
        click.echo(f"Adding template {filename}...")
    with open(filename) as f:
        data = str(f.read())

        with open(params) as p:
            params = json.loads(str(p.read()))
            name = filename.split('/')[-1]
            name = '-'.join(name.split('.')[0:-1])
            if tag is None:
                tag = name
            ret = client.update_on_conflict(
                table='experiments',
                record={
                    'filename': name,
                    'data': json.dumps(data),
                    'description': f"Template {name}",
                    'type': 'template',
                    'profile': params,
                    'last_update_time': str(datetime.now()),
                    'duration': params.get('duration', 30),
                    'tag': tag
                },
                conflicting_cols=['filename', 'type']
            )
        if ret is not None:
            click.echo("Template loaded")
        else:
            click.echo("Error loading template")

@cli.command(help="Retrieves the log for that session/log_type")
@click.option('--exp_id', required=True, help='the experiment id')
@click.option('--log_type', default='1', help='the log type')
def get_exp_log (exp_id, log_type):
    data = client.select(
        table='exp_logs',
        fetch=['log_text'],
        filter={
            'log_type': log_type,
            'exp_id': exp_id
        },
        sort='creation_time desc',
        limit=1)
    if data is None:
        click.echo("Error getting experiment {} log".format(exp_id))
    else:
        pp.pprint(data)

@cli.command(help="List of today's planned sessions")
def list_today_sessions():
    today = date.today()
    dom = today.day
    month = today.month
    list_sessions_short([
        "dom:{:02d}".format(today.day),
        "month:{:02d}".format(today.month)
        ])

@cli.command(help="List of yesterday's planned sessions")
def list_yesterday_sessions():
    yesterday = date.fromtimestamp(time.time() - 86400)
    list_sessions_short([
        "dom:{:02d}".format(yesterday.day),
        "month:{:02d}".format(yesterday.month)
        ])

@cli.command(help="Show all test campaigns")
def list_campaigns():
    cols=['id', 'title', 'status', 'resources', 'experiments']
    data = client.select(
        table='test_campaigns',
        fetch=cols,
        sort='id',
        limit=100
    )
    for row in data:
        row['resources']=pp_col.pformat(row['resources'])
        row['experiments']=pp_col.pformat(row['experiments'])
    click.echo(tabulate(data, headers='keys'))

@cli.command(help="List all sessions attributes matching a filter")
@click.argument('filter', nargs=-1)
def list_sessions_long(filter):
    for arg in filter:
        if ':' not in arg:
            click.echo('Filters are "key":"value" pairs')
            exit()
    d = dict(arg.split(':') for arg in filter)
    data = client.get_sessions(**d)
    click.echo(tabulate(data, headers='keys'))

@cli.command(help="Lists a few sessions attributes using a filter")
@click.argument('filter', nargs=-1)
def list_sessions_short(filter):
    for arg in filter:
        if ':' not in arg:
            click.echo('Filters are "key":"value" pairs')
            exit()
    d = dict(arg.split(':') for arg in filter)
    data = client.get_sessions(**d)
    for exp in data:
        for key in exp.copy():
            if key not in [
                    'exp_id', 
                    'filepath', 
                    'mins',
                    'hour',
                    'dom',
                    'dow',
                    'month',
                    'year',
                    'campaign',
                    'status']:
                del exp[key]
    data_sorted = sorted(data, key=lambda data: data['dom'])
    click.echo(tabulate(data_sorted, headers='keys'))

@cli.command(help="Lists a very few sessions attributes using a filter")
@click.argument('filter', nargs=-1)
def list_sessions_very_short(filter):
    for arg in filter:
        if ':' not in arg:
            click.echo('Filters are "key":"value" pairs')
            exit()
    d = dict(arg.split(':') for arg in filter)
    data = client.get_sessions(**d)
    for exp in data:
        for key in exp.copy():
            if key not in [
                    'exp_id', 
                    'mins',
                    'hour',
                    'dom',
                    'month',
                    'year',
                    'campaign',
                    'status']:
                del exp[key]
        p=re.compile('(-|\w*)*-int-nft')
        m=p.search(exp['exp_id'])
        exp['exp_id']=m.group(0)
    data_sorted = sorted(data, key=lambda data: data['dom'])
    click.echo(tabulate(data_sorted, headers='keys'))


@cli.command(help="Executes a sql command")
@click.option('--cmd', required=True, help='the sql command')
@click.option('--attrs', help='the command attributes as json')
def execute_command(cmd, attrs):
    if attrs is None:
        attrs = '[]'
    res = client.execute_command( cmd, json.loads(attrs))
    pp.pprint(res)

@cli.command(help="Get the team behind a deployment")
@click.option('--name', required=True, help='the deployment name')
@click.option('--context', required=True, help='the context')
def get_team_of_deployment(name, context):
    res = client.execute_command( 
        """select profile->'labels'->>'team' as team from k8s_resources 
            where kind = 'deployment' and name = %s and context = %s""", [name, context])
    pp.pprint(res)

@cli.command(help="Get the deployments of a namespace")
@click.option('--name', required=True, help='the namespace name')
@click.option('--context', required=True, help='the context')
def get_deployments_of_namespace(name, context):
    res = client.execute_command( 
        """select name from k8s_resources 
            where kind = 'deployment' and context = %s and 
            parent_id in (select id from k8s_resources where kind = 'namespace' and
            context = %s and name = %s)
        """, [context, context, name])
    pp.pprint(res)

if __name__ == "__main__":
    cli()
